# This is a module for the GAP package MAPCLASS
# computing the orbit of the mapping class group. 
# This module contains various utility functions. 
# 
# A. James, K. Magaard, S. Shpectorov 2011


#
# NumberOfCommutatorProducts
# function for computing the number of ways to write a given 
# group element as a product of n commutators. 
# Reference: R. Guralnick's thesis
# t=table, c= index of some conjugacy class wrt t, n= number of commutators
InstallGlobalFunction(NumberOfCommutatorProducts,function(t,c,n) 
 
 return SizesCentralizers(t)[1]^(2*n-1)*
        Sum(List(Irr(t),x->x[c]/(x[1]^(2*n-1))));

end);
  

#
# TotalNumberOfTuples
# function computing the structure constant
# t = character table
# g = genus
# cc = the principal tuple in terms of it character table ccs
#
InstallGlobalFunction(TotalNumberOfTuples,function(t,g,cc)
 local i,s;

 if Length(cc)=0 then
  return NumberOfCommutatorProducts(t,1,g);
 fi;

 s:=0;
 for i in [1..Length(SizesCentralizers(t))] do
  s:=s+NumberOfCommutatorProducts(t,i,g)*
       ClassStructureCharTable(t,Concatenation([InverseClasses(t)[i]],cc));
 od;
 return s;

end);

#
# TotalNumberTuples
# Computes the structure constant using group data
InstallGlobalFunction(TotalNumberTuples, function(group, tuple, genus)
  local tups, tbl, cc;

  tbl:=CharacterTable(group);
  cc:=ConjugacyClasses(tbl);
  tups:=List(tuple, i -> Filtered([1..Length(cc)], x -> i in cc[x])[1]);
  return TotalNumberOfTuples(tbl, genus, tups);
end);
  
# End
